import serial
import ConfigParser
import os
import time

import logging

class NullHandler(logging.Handler):
    def emit(self, record):
        pass

# XXX: platform-dependent, but Pyserial doesn't offer Serial.readline with eol
# argument anymore for python >= 2.6 because io.RawIOBase doesn't implement it.
class MySerial(serial.PosixSerial, serial.FileLike):
    pass

h = NullHandler()
logging.getLogger("GPibSerial").addHandler(h)

# options
CONFIGFILES = ['/etc/gpibserial.conf',os.path.expanduser('~/.gpibserial.conf'), 'gpibserial.conf']

# Create ConfigParser object and read the config file
# configfiles: list of strings with name/path of configfiles, will be read in
# that order
# returns: ConfigParser
# raises IOError if none of the configfiles were found
def _read_config(configfiles=CONFIGFILES):
    # search for serial port data in /etc/gpibserail.conf, ~/.gpibserial.conf or ./gpibserial.conf
    config = ConfigParser.ConfigParser()
    if len(config.read(CONFIGFILES)) == 0:
        raise IOError('No config file found: '+', '.join(configfiles))
    return config

def list_devices():
    config = _read_config()
    return config.sections()

# Find serial port settings for device and open the port with these settings
# devname: name of section in config file
# returns: serial object
# raises ValueError if devname is no valid section or misses mandatory
# parameters, IOError if none of the configfiles were found, and whatever the
# serial class raises.
def find(devname):
    bytesize_values = { '5':serial.FIVEBITS, '6':serial.SIXBITS, '7':serial.SEVENBITS, '8':serial.EIGHTBITS }
    parity_values = { 'N':serial.PARITY_NONE, 'E':serial.PARITY_EVEN, 'O':serial.PARITY_ODD }
    stopbits_values = { '1':serial.STOPBITS_ONE, '2':serial.STOPBITS_TWO }

    config = _read_config()
    
    # read port (mandatory)
    try:
        port = config.get(devname,'port')
    except ConfigParser.NoSectionError:
        raise ValueError('Section not found: %s' % (devname,))
    except ConfigParser.NoOptionError:
        raise ValueError('Section %s has no port option' % (devname,))
    ser = MySerial()
    ser.port = port
    # Add custom attribute (not used by serial)
    ser.eol = '\n'

    # optional options
    for option in config.options(devname):
        if option == 'baudrate':
            ser.baudrate = config.getint(devname,'baudrate')
        elif option == 'bytesize':
            ser.bytesize = bytesize_values[config.get(devname,'bytesize')]
        elif option == 'parity':
            ser.parity = parity_values[config.get(devname,'parity')]
        elif option == 'stopbits':
            ser.stopbits = stopbits_values[config.get(devname,'stopbits')]
        elif option == 'timeout':
            ser.timeout = config.getfloat(devname,'timeout')
        elif option == 'xonxoff':
            ser.xonxoff = config.getboolean(devname,'xonxoff')
        elif option == 'rtscts':
            ser.rtscts = config.getboolean(devname,'rtscts')
        elif option == 'eol':
            # parse backslash escapes
            ser.eol = eval('"%s"' % (config.get(devname,'eol'),))

    # XXX: handle errors
    ser.open()
    return ser

# Clear buffers (RS232 BREAK is equivalent to GPIB DCL on a tek scope)
def clear(port):
    port.sendBreak(duration=0.25)
    time.sleep(0.25)
    data = ''
    while not data.startswith('DCL'):
        data = port.readline(size=10, eol=port.eol)
        
# Write command
# The scopes always expect a CRLF as EOL, regardless of the EOL settings in the
# RS232 menu
def write(port,data):
    logging.debug('WRITE: %s', data)
    port.write(data+'\r\n')

# Read a line, terminated by the EOL symbol(s)
def read(port,maxlen):
    data = port.readline(size=maxlen, eol=port.eol)
    logging.debug('READ: %s', data)
    return data

# Read GPIB block, in the form: #3123(123 bytes)
def readbin(port,maxlen):
    data = ''
    n = 0
    # The TDS3k first sends a single quote and a newline for some reason
    # skip up to 10 characters until a # is found
    while data != '#':
        data = port.read(size=1)
        n += 1
        if n > 10:
            raise IOError('Waited 10 chars, still no #')
    # number of digits for length
    lengthdigits = int(port.read(size=1))
    # actual length
    length = int(port.read(size=lengthdigits))
    # actual data
    data = port.read(size=length+len(port.eol))
    logging.debug('READBLOCK: %d bytes', len(data))
    return data

# Read until no data is received for 5 seconds
# XXX: find real way to detect end of hardcopy (interpret BMP/PNG header? serial
# status?
# Inittimeout is the timeout for the initial byte, timeout for the rest of the
# bytes
def readraw(port, inittimeout=10, timeout=5):
    data = ''
    c = 'X'
    oldtimeout = port.timeout
    port.timeout = inittimeout
    data = port.read(size=1)
    port.timeout = timeout
    while c != '':
        c = port.read(size=1)
        data += c
    port.timeout = oldtimeout
    logging.debug('READBIN: %d bytes', len(data))
    return data

# Wait until a byte is received to start the transmission. It polls for interval
# seconds and tries again until a byte is detected, then the byte is returned.

# The point of the timeout is that the software courd theoretically take control
# after each timeout.
def waitforbyte(port,interval=10):
    oldtimeout = port.timeout
    port.timeout = interval
    data = ''
    while data == '':
        data = port.read(size=1)
    port.timeout = oldtimeout
    return data
